#include "BSD_Communicator.h"

/********************************************************
 * Allocate the socket (endpoint), setting myDescriptor.
 * (polymorphic, called from Communicator::Open()).
 * PRE: all data members except myDescriptor are initialized.
 * POST: myDescriptor has been initialized (an endpoint
 *       has been allocated).
 ********************************************************/
void BSD_Communicator::allocate()
{
  myDescriptor = std::socket(PF_INET,
			protocolInfo()->BSD_Info(),
			protocolInfo()->number());
  if (myDescriptor < 0)
    fatal("BSD_Communicator::allocate(): call to socket failed");
}


/**********************************************************
 * receive a message via an arbitrary descriptor (utility).
 *  Could block.
 *   See note about TCP/UDP client/servers under the 
 *   send(const string & msg) documentation.
 * PRE: I am open && someone is sending me a message
 *       via descriptor.
 * POST: message has been received in msg && 
 *        sender contains the address of the sender &&
 *        lastSender is a copy of sender &&
 *        RV == the number of bytes received.
 **********************************************************/
int BSD_Communicator::receive(string & msg)
{
  const int BUF_SIZE = 1024;
  char buf[BUF_SIZE];
  msg = "";
  int total = 0;
  int count;
  // Loop until we have received the whole message.
  do
    {
      count = std::recv(getDescriptor(), buf, BUF_SIZE, 0);
      buf[count] = '\0';
      msg += buf;
      total += count;
    }
  while (count >= BUF_SIZE);

  return total;
}


/******************************************************
 * Address mutator
 * PRE: address is a pointer to a SocketAddress object.
 * POST: myAddressInfo == address.
 ******************************************************/
void BSD_Communicator::setAddressInfo(SocketAddress * address)
{
  if (myAddressInfo != NULL)
    delete myAddressInfo;

  myAddressInfo = address;
}

/***************************************************
 * compare() utility function
 * (polymorphic, used by relational ops).
 * PRE: netObj is a BSD_Communicator object.
 * POST: RV == 0 if we are the same BSD_Communicator,
 *            +X if I am > than netObj, and
 *            -X if I am < netObj
 *            (using descriptor values).
 ***************************************************/

int BSD_Communicator::compare(const NetworkObject & netObj) const
{
  const BSD_Communicator * ptr = dynamic_cast<const BSD_Communicator*>(&netObj);
  if (ptr == NULL) fatal("BSD_Communicator::compare");

  return getDescriptor() - ptr->getDescriptor();
}

/***************************************************
 * print() utility.
 * PRE: out is an open ostream.
 * POST: my description has been inserted into out.
 ***************************************************/
void BSD_Communicator::print(ostream & out) const
{
  Communicator::print(out);
  if (myAddressInfo) {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  << (*myAddressInfo);
  } else {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  <<  "(no address)";
  }
}


/***************************************************
 * operator= - implement the assignment operator
 * PRE: comm is a class which inherits BSD_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
BSD_Communicator & BSD_Communicator::operator=(const BSD_Communicator & comm)
{
   Communicator::operator=(comm);
   this->myDescriptor = comm.myDescriptor;
   this->myAddressInfo = comm.myAddressInfo;
   return *this;
}

/***************************************************
 * Copy Constructor
 * PRE: comm is a class which inherits BSD_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
BSD_Communicator::BSD_Communicator(const BSD_Communicator & comm)
   : Communicator(comm)
{
   this->myDescriptor = comm.myDescriptor;
   *(this->myAddressInfo) = *(comm.myAddressInfo);
}
