/*************************************************************
 * Pthread.cpp -- implementation of PosixThread in a box
 *
 * Author: Ryan VanderBijl
 * Date:   May/June 2000
 */

#include "PosixThread.h"
#include <iostream>
#include <time.h>
#include <errno.h>

#ifndef WIN32
#include <unistd.h>
#endif

/****************************************************
 * Constructor
 *  Allocates all the information mutex's and 
 *  condition variables needed. It also creates 
 *  the thread, and causes it to wait for this classes's
 *  start() function to be called.
 * POST: the thread is created, and is waitting to 
 *       be told to work.
 ****************************************************/
PosixThread::PosixThread() {
   /* create/initiate the attr, cond, and mutex.. */
   pthread_attr_t myAttr;

   myBusy = false;

   /* Attributes: (make it detatched) */
   if (pthread_attr_init(&myAttr)) { fatal("In PosixThread Creation:"); }
   if (pthread_attr_setdetachstate(&myAttr, PTHREAD_CREATE_DETACHED))
       { fatal("In PosixThread Creation:"); }

   /* Condition: (make it shared through the process) */
   if (pthread_cond_init(&myCond, NULL)) 
       { fatal("In PosixThread Creation:"); }

   /* Mutex: make some plain ol' normal mutex */
   if (pthread_mutex_init(&myWaitMutex, NULL))
       { fatal("In PosixThread Creation:"); }
   if (pthread_mutex_init(&myAccessorMutex, NULL))
       { fatal("In PosixThread Creation:"); }

   /* create the new thread */
   if (pthread_create(&myPosixThreadID, &myAttr, _sleeper, (void*)this))
       { fatal("In PosixThread Creation:"); }

   /* dont need the attribute stuff anymore */
   if (pthread_attr_destroy(&myAttr)) // dont need it any more
       { fatal("In PosixThread Creation:"); }

   /* Now we want to wait on the child thread to make sure it
    *  is completely ready before we have any chance of calling
    *  its start() 
    */
#ifdef WIN32
   Sleep(250);  //Windows doesn't have usleep, but this provides
		//  a way to sleep for milliseconds. We sleep 1/4 sec.
#else
   usleep(250000);  // for unix systems.
#endif /* WIN32 */
}



/***************************************************
 * Deconstructor
 *   destory the condition and mutex variables,
 *   and then cancels the thread.
 ***************************************************/
PosixThread::~PosixThread() {
   /* destroy the condion variable stuff */
   if (pthread_cond_destroy(&myCond)) { perror("error killing thread");}

   /* destory the <plural>mutex</plural> */
   if (pthread_mutex_destroy(&myWaitMutex))
       { perror("error killing thread");}
   if (pthread_mutex_destroy(&myAccessorMutex))
       { perror("error removing mutex in constructor:");}

   /* cancel execution of the thread! */
   pthread_cancel(threadID());
}

/*************************************
 *  isBusy -- a protected way to 
 *    access wether or not we are busy
 *  Should be const, but b/c it changes
 *  the mutex's, it can't be.
 *************************************/
bool PosixThread::isBusy()
{
   bool val;
   if (pthread_mutex_lock(&myAccessorMutex)) 
       { this->fatal("trying to lock, isBusy:");}
   val = myBusy;
   if (pthread_mutex_unlock(&myAccessorMutex))
       { this->fatal("trying to unlock, isBusy:");}
   return val;
}

/*************************************
 *  setBusy -- a protected way to 
 *    access the myBusy stuff.
 *************************************/
bool PosixThread::setBusy(bool new_status)
{
   if (pthread_mutex_lock(&myAccessorMutex))
       { this->fatal("trying to lock, setBusy:");}
   myBusy = new_status;
   if (pthread_mutex_unlock(&myAccessorMutex))
       { this->fatal("trying to unlock, setBusy:");}
   return new_status;
}



/***************************************************
 * start() - called by the MAIN thread to tell the
 *  the thread to start running the run function,
 *  which is should be defined in a class derived
 *  from this.
 * PRE: the thread is not busy, and is waitting for
 *      a job. (pthread_cond_wait in the _sleepr function)
 * POST: the thread is busy, running the run function
 ***************************************************/
inline void PosixThread::start() {
   if (pthread_mutex_lock(&this->myWaitMutex))
       { fatal("In PosixThread start: eror locking mutex"); }

   /* Inside the lock, to make sure that we really can 
    *  send this thread a message, and have it realize
    *  it is busy. 
    */
   this->setBusy(1);

   if (pthread_cond_signal(&myCond))
       { fatal("In PosixThread start: error sending signal"); }

   if (pthread_mutex_unlock(&this->myWaitMutex))
       { fatal("In PosixThread start: eror unlocking mutex"); }
}


/***************************************************
 * _sleeper: the magic static function
 *   This function "contains" the thread: it is
 *   what is called when the thread is created.
 *   This sits, doing nothing, waitting to receive
 *   a signal -- which is generated by start(). Then
 *   it runs the run() function, defined in a 
 *   derived class, and then waits some more.
 * This is a static function, and thus may not use
 *  'this' as a variable. So we cheat. we pass 'this'
 *  as a pointer through pthread_create, and keep
 *  'this' in 'self'. 
 *
 * Some credit due to: Ian Emmons, and some anonymous
 * guy. Info found on the threading FAQ, found here:
 *    http://www.lambdacs.com/newsgroup/FAQ.html#Q63
 ***************************************************/
void *PosixThread::_sleeper(void *arg) {
   PosixThread *self = (PosixThread*)arg; 

   //the cond_wait unlocks the mutex, and reaquires it again when
   //  it returns. Meanning that we get full control until we are
   //  waitting again.
   pthread_mutex_lock(&self->myWaitMutex);

   while(1) { 
       // not busy, so we wait until we have a job and are busy
       self->setBusy(0);
       while(! self->isBusy() ) {
	   //the wait unlocks the mutex
	   if (pthread_cond_wait(&self->myCond,&self->myWaitMutex) )
	       self->fatal("trying to wait:");
       }

       //time to goto work!
       self->run();
   }
}
