/* BSD_Communicator.h extends the Communicator abstraction with functionality
 *   specific to BSD sockets.
 *
 * Author: Joel Adams.
 * Date: November, 1998.
 * Version: 0.9
 *
 * Modified: Ryan VanderBijl
 * Date:     May/June, 2000
 *
 */

#ifndef BSD_COMMUNICATOR
#define BSD_COMMUNICATOR

#include <netinet/in.h>       // ntohl(), ntohs(), htons(), htonl()
#include "Communicator.h"
#include "SocketAddress.h"
#include <unistd.h>
using namespace std;

class BSD_Communicator : public Communicator
{
 public:
  BSD_Communicator();
  BSD_Communicator(const BSD_Communicator & comm);
  virtual ~BSD_Communicator();

  void start(Service * service);
  virtual void stop();

  virtual int send(const string & msg);
  virtual int receive(string & msg);


  SocketAddress * addressInfo() const;

  virtual int compare(const NetworkObject & netObj) const;
  virtual void print(ostream & out) const;
  virtual string className() const;
  BSD_Communicator & operator=(const BSD_Communicator & comm);

 protected:
  virtual int getDescriptor() const;

  virtual void allocate();
  void setAddressInfo(SocketAddress * address);

 private:
  int myDescriptor;             // endpoint for BSD sockets
  SocketAddress * myAddressInfo;
};


/********************************************************
 * Default constructor.
 * POST: all data members have been initialized
 *        to default values.
 ********************************************************/

inline BSD_Communicator::BSD_Communicator()
  : Communicator()
{
  myDescriptor = -1;
  myAddressInfo = NULL;
}


/********************************************************
 * Open for a given service.
 * PRE: service points to a properly-defined Service.
 * POST: myServiceInfo points to service.
 ********************************************************/

inline void BSD_Communicator::start(Service * service)
{
  Communicator::start(service);
}

/*********************************************************
 * descriptor/endpoint accessor.
 * POST: RV == myDescriptor.
 *********************************************************/

inline int BSD_Communicator::getDescriptor() const
{
  return myDescriptor;
}

/**********************************************
 * stop(close) the socket.
 * POST: I have been stopped, and all of my
 *     dynamically-allocated memory has been reclaimed.
 **********************************************/
inline void BSD_Communicator::stop()
{
  Communicator::stop();
  delete myAddressInfo;
  std::close(getDescriptor());
}

/**************************************************
 * Destructor.
 * POST: If I was started, then I am stopped.
 **************************************************/

inline BSD_Communicator::~BSD_Communicator()
{
  if (isOn())
    stop();
}


/**********************************************************
 * Send message to remote address that sent to me most recently.
 *  Could block.
 *
 * This works for TCP and UDP clients, and TCP server (although
 *     for TCP server the getDescriptor() is overloaded).
 *     UDP Server needs to overload receive and this function so
 *     that we use sendto and recvfrom. (UDP Client works like this
 *     because the socket is already connected to the server.)
 *
 * PRE: I am started && msg is a reply to my most recent sender.
 * POST: message has been sent && RV == the number of bytes sent.
 **********************************************************/
inline int BSD_Communicator::send(const string & msg)
{
  return std::send(getDescriptor(), msg.c_str(), msg.length(), 0);
}


/******************************************************
 * SocketAddress accessor
 * POST: RV == myAddressInfo.
 ******************************************************/
inline SocketAddress * BSD_Communicator::addressInfo() const
{
  return myAddressInfo;
}


/***************************************************
 * Class className accessor.
 * POST: RV == BSD_Communicator.
 ***************************************************/

inline string BSD_Communicator::className() const
{
  return "BSD_Communicator";
}


#endif
