/* Host.h provides a simple abstraction for IP hosts.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.8
 */

#ifndef _HOST_
#define _HOST_

#ifndef WIN32
#include <netdb.h>                   // getprotobyname(), protoent
#include <arpa/inet.h>               // inet_ntoa()
#else
#include <winsock2.h>
#endif /* WIN32 */

#include "NetworkObject.h"
using namespace std;

class Host : public NetworkObject
{
public:
  Host();
  Host(const string & name);
  Host(const struct in_addr & ipAddress);
  virtual ~Host();
  Host(const Host & host);

  string name() const;
  unsigned addressNBO() const;
  string addressString() const;

  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & netObj) const;
  virtual string className() const;
  Host &operator=(const Host & host);

protected:

private:
  string myName;              // host name
  struct in_addr myAddress;   // IP address (network byte order)
  string myAddressString;     // IP address (dotted notation)
};


/*****************************************************************
 * Copy Constructor
 * PRE: host is an Host object 
 * POST: data members are equal to host's data memebers.
 *****************************************************************/
inline Host::Host(const Host & host)
{
   this->myName = host.myName;
   this->myAddressString = host.myAddressString;
   ::memcpy(&this->myAddress, &host.myAddress, sizeof(this->myAddress));
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: host is an Host object 
 * POST: data members are equal to host's data memebers.
 *****************************************************************/
inline Host &Host::operator=(const Host & host)
{
   this->myName = host.myName;
   this->myAddressString = host.myAddressString;
   ::memcpy(&this->myAddress, &host.myAddress, sizeof(this->myAddress));
   return *this;
}



/**********************************************
 * Default constructor
 * POST: members have received default values.
 **********************************************/

inline Host::Host()
{
  memset(&myAddress, 0, sizeof(struct in_addr));
}



/*************************************************
 * Destructor
 * Nothing to do.
 *************************************************/

inline Host::~Host()
{}

/*************************************************
 * Name accessor.
 * POST: RV == my name.
 *************************************************/

inline string Host::name() const
{
  return myName;
}

/*************************************************
 * IP address accessor (dotted notation)
 * PRE: I have a valid IP address.
 * POST: RV == my IP address (dotted notation)
 *************************************************/

inline string Host::addressString() const
{
  return myAddressString;
}


/*************************************************
 * IP address accessor (network byte-order)
 * PRE: I have a valid IP address.
 * POST: RV == my IP address (long, network byte-order form)
 *************************************************/

inline unsigned Host::addressNBO() const
{
  return myAddress.s_addr;
}


/**************************************************
 * (Polymorphic print(), called by operator<<)
 * PRE: out is an ostream.
 * POST: out contains my host information.
 **************************************************/

inline void Host::print(ostream & out) const
{
  out << "Host: " << name() << " (" << addressString() << ')';
}


/****************************************************
 * (Polymorphic) className() method used by diagnostics
 * POST: RV == "Host".
 ****************************************************/

inline string Host::className() const
{
  return "Host";
}

#endif
