/* NetworkObject.h defines a root class for the
 *  various sub-hierarchies (for storing common functionality).
 *
 * Author: Joel C. Adams.
 * Date: October, 1998.
 * Version: 0.9
 */

#ifndef NETWORK_OBJECT
#define NETWORK_OBJECT

#include <string>             // C++ string class
#include <iostream>           // C++ stream I/O
#include <cstdio>             // vfprintf()
#include <cstdarg>            // ... mechanism
#include <cstdlib>            // exit()
#include <errno.h>            // perror(), ...

#ifndef WIN32
#include <sys/socket.h>  //neeeded for AF_INET, in a Host, NetworkAddres
#endif /* WIN32 */

using namespace std;

class NetworkObject
{
 public:
  virtual void print(ostream & out) const;
#ifndef WIN32
  friend ostream & operator<<(ostream & out, 
			      const NetworkObject & netObj);
#endif /* WIN32 */

  virtual int compare(const NetworkObject & netObj) const = 0;
  bool operator==(const NetworkObject & netObj) const;
  bool operator!=(const NetworkObject & netObj) const;
  bool operator<=(const NetworkObject & netObj) const;
  bool operator>=(const NetworkObject & netObj) const;
  bool operator<(const NetworkObject & netObj) const;
  bool operator>(const NetworkObject & netObj) const;

  virtual string className() const = 0;
  virtual ~NetworkObject();
  
 protected:
  NetworkObject();
  virtual void fatal(const char * format, ...) const;
  virtual void warning(const char * format, ...) const;

 private:
};


/******************************************************
 * Default constructor (nothing to do)
 ******************************************************/

inline NetworkObject::NetworkObject()
{}


/************************************************
 * Destructor (not much to do)
 ************************************************/

inline NetworkObject::~NetworkObject()
{}

/******************************************************
 * (Polymorphic) print function (used by operator<<)
 * PRE: out is an open ostream.
 * POST: out contains my class information
 ******************************************************/

inline void NetworkObject::print(ostream & out) const
{
  out << "NetworkObject";
}


/******************************************************
 * (Polymorphic) print class information
 * PRE: stream is the name of a valid output stream.
 * POST: stream contains class information.
 ******************************************************/

inline ostream & operator<<(ostream & out, 
			    const NetworkObject & netObj)
{
  netObj.print(out);
  return out;
}



/**************************************************
 * Equality
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I == netObj.
 **************************************************/

inline bool NetworkObject::operator==(const NetworkObject & netObj) const
{
  return !compare(netObj);  // polymorphic, defined by subclasses
}

/**************************************************
 * Inequality
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I != netObj.
 **************************************************/

inline bool NetworkObject::operator!=(const NetworkObject & netObj) const
{
  return compare(netObj) != 0;  // polymorphic, defined by subclasses
}

/**************************************************
 * Less-than-or-equal
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I <= netObj.
 **************************************************/

inline bool NetworkObject::operator<=(const NetworkObject & netObj) const
{
  return compare(netObj) <= 0;  // polymorphic, defined by subclasses
}

/**************************************************
 * Greater-than-or-equal
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I >= netObj.
 **************************************************/

inline bool NetworkObject::operator>=(const NetworkObject & netObj) const
{
  return compare(netObj) >= 0;  // polymorphic, defined by subclasses
}

/**************************************************
 * Less-than
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I < netObj
 **************************************************/

inline bool NetworkObject::operator<(const NetworkObject & netObj) const
{
  return compare(netObj) < 0;  // polymorphic, defined by subclasses
}

/**************************************************
 * Greater
 * PRE: netObj is an object we wish to compare
 * POST: RV == true iff I > netObj
 **************************************************/

inline bool NetworkObject::operator>(const NetworkObject & netObj) const
{
  return compare(netObj) > 0;  // polymorphic, defined by subclasses
}


#endif


