/* TCP_Client_TLI_Communicator.h defines the Communicator abstraction
 *  for TCP client sockets under TLI.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.9
 *
 * Modified: Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef TCP_CLIENT_TLI_TRANSPORTER
#define TCP_CLIENT_TLI_TRANSPORTER

#include "Client_TLI_Communicator.h"
#include "TCP_Service.h"
#include "TLI_TCP_SendRecv.h"
using namespace std;

class TCP_Client_TLI_Communicator : public TLI_TCP_SendRecv,
				    public Client_TLI_Communicator
{
 public:
  TCP_Client_TLI_Communicator();

  TCP_Client_TLI_Communicator(const string & serviceName,
			const string & remoteHostName);
  TCP_Client_TLI_Communicator(const unsigned short port,
			const string & remoteHostName);
  TCP_Client_TLI_Communicator(const string & serviceName, 
			const unsigned short port,
			const string & remoteHostName);
  TCP_Client_TLI_Communicator(const TCP_Client_TLI_Communicator & comm);

  void open(const string & serviceName,
	    const string & remoteHostName);
  void open(const unsigned short port,
	    const string & remoteHostName);
  void open(const string & serviceName, 
	    const unsigned short port,
	    const string & remoteHostName);
  void close();

  virtual void connect();
  int receive(string & msg);
  int send(const string & msg);

  virtual ~TCP_Client_TLI_Communicator();

  virtual string className() const;
  TCP_Client_TLI_Communicator &
      operator=(const TCP_Client_TLI_Communicator & comm);

 protected:
 private:
};


/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from TCP_Client_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline TCP_Client_TLI_Communicator::TCP_Client_TLI_Communicator(const TCP_Client_TLI_Communicator & comm)
    : Client_TLI_Communicator(comm)
{
}


/****************************************************
 * Default constructor
 * POST: Inherited data members have default values.
 ****************************************************/

inline TCP_Client_TLI_Communicator::TCP_Client_TLI_Communicator()
  : Client_TLI_Communicator()
{}


/***************************************************
 * open with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a TCP_Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/
inline void TCP_Client_TLI_Communicator::open(const string & serviceName,
					const string & hostName)
{
  Client_TLI_Communicator::open(new TCP_Service(serviceName), hostName);
}

/***************************************************
 * Construct with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to the Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/

inline TCP_Client_TLI_Communicator::
TCP_Client_TLI_Communicator(const string & serviceName,
		      const string & hostName)
  : Client_TLI_Communicator()
{
  open(serviceName, hostName);
}


/***************************************************
 * open with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to the TCP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/
inline void TCP_Client_TLI_Communicator::open(const unsigned short port,
					const string & hostName)
{
  Client_TLI_Communicator::open(new TCP_Service(port), hostName);
}


/***************************************************
 * Construct with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/
inline TCP_Client_TLI_Communicator::
TCP_Client_TLI_Communicator(const unsigned short port,
		      const string & hostName)
  : Client_TLI_Communicator()
{
  open(port, hostName);
}


/***************************************************
 * open with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline void TCP_Client_TLI_Communicator::open(const string & serviceName, 
					const unsigned short port,
					const string & hostName)
{
  Client_TLI_Communicator::open(new TCP_Service(serviceName, port), 
			  hostName);
}



/***************************************************
 * Construct with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline TCP_Client_TLI_Communicator::
TCP_Client_TLI_Communicator(const string & serviceName, 
		      const unsigned short port,
		      const string & hostName)
  : Client_TLI_Communicator()
{
  open(serviceName, port, hostName);
}

/***************************************************
 * close the socket 
 * (supplied for completeness, should be optimized away)
 * POST: I have been closed.
 ***************************************************/
inline void TCP_Client_TLI_Communicator::close()
{
  Client_TLI_Communicator::close();
}


/***************************************************
 * Destructor 
 * POST: I have been closed.
 ***************************************************/
inline TCP_Client_TLI_Communicator::~TCP_Client_TLI_Communicator()
{
  if (isOn())
    close();
}


/***************************************************
 * Class className accessor
 * POST: RV == "TCP_Client_TLI_Communicator".
 ***************************************************/
inline string TCP_Client_TLI_Communicator::className() const
{
  return "TCP_Client_TLI_Communicator";
}


/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from Client_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline TCP_Client_TLI_Communicator &TCP_Client_TLI_Communicator::operator=(const TCP_Client_TLI_Communicator & comm)
{
    Client_TLI_Communicator::operator=(comm);
    return *this;
}


/*****************************************************************
 * send -- send a message to the server
 *  This call blocks until the message is sent.
 *  
 * PRE:  msg contains the message to send.
 *       the communicator has been connected 
 * POST: returns value of t_snd:
 *         -1 if there was an error,
 *         otherwise the number of characters received by the
 *           server
 *****************************************************************/
inline int TCP_Client_TLI_Communicator::send(const string & msg) 
{
  return TLI_TCP_SendRecv::send(getDescriptor(),msg);
}


/*****************************************************************
 * receive - receive a message from the server
 *  This call blocks until the message a message is receieved.
 * PRE: this communicator has been connected.
 * POST: msg contains the message sent from the server.
 *       Returns the number of characters received, or 
 *       -1 if there was an error.
 *****************************************************************/
inline int TCP_Client_TLI_Communicator::receive(string & msg) 
{
  return TLI_TCP_SendRecv::receive(getDescriptor(),msg);
}

#endif
