/* TCP_Server_BSD_Communicator.h defines a Communicator for connection-oriented
 *  TCP/IP Servers using BSD sockets.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 *
 * Modified by: Ryan VanderBijl
 * Date:        May/June 2000
 */

#ifndef TCP_SERVER_BSD_TRANSPORTER
#define TCP_SERVER_BSD_TRANSPORTER

#include "Server_BSD_Communicator.h"
#include "TCP_Service.h"
using namespace std;

class TCP_Server_BSD_Communicator : public Server_BSD_Communicator
{
 public:
  enum { DEFAULT_QUEUE_LENGTH = 5 };

  TCP_Server_BSD_Communicator(const int queueLength = DEFAULT_QUEUE_LENGTH);
  TCP_Server_BSD_Communicator(const string & service, 
			     const unsigned short port,
			     const int queueLength = DEFAULT_QUEUE_LENGTH);
  TCP_Server_BSD_Communicator(const TCP_Server_BSD_Communicator & comm);

  void open(const string & service, const unsigned short port);
  void close();
  virtual ~TCP_Server_BSD_Communicator();

  virtual void acceptConnection();
  virtual void closeConnection();
  virtual void forgetConnection();

  virtual string className() const;
  virtual void print(ostream & out) const;
  TCP_Server_BSD_Communicator & 
      operator=(const TCP_Server_BSD_Communicator & comm);
  
 protected:
  virtual int getDescriptor() const; //overload the communicator
  void setQueueLength(int length);
  void listen();

 private:
  int myConnectionDescriptor;
  int myQueueLength;
};


/*****************************************************************
 * Copy constructor
 * PRE: comm is an object which inherits TCP_Server_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline TCP_Server_BSD_Communicator::TCP_Server_BSD_Communicator(const TCP_Server_BSD_Communicator & comm)
   : Server_BSD_Communicator(comm)
{
  this->myConnectionDescriptor = comm.myConnectionDescriptor;
  this->myQueueLength = comm.myQueueLength;
}


/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits TCP_Server_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline TCP_Server_BSD_Communicator& TCP_Server_BSD_Communicator::operator=(const TCP_Server_BSD_Communicator & comm)
{
  Server_BSD_Communicator::operator=(comm);
  this->myConnectionDescriptor = comm.myConnectionDescriptor;
  this->myQueueLength = comm.myQueueLength;
  return *this;
}

/*****************************************************************
 * Default constructor 
 * PRE: queueLength > 0.
 * POST: myQueueLength == queueLength.
 *****************************************************************/
inline TCP_Server_BSD_Communicator::TCP_Server_BSD_Communicator(int queueLength)
  : Server_BSD_Communicator()
{
  setQueueLength(queueLength);
}


/*****************************************************************
 * Listen for connection.
 * PRE: I have been initialized and bound to a Service or port.
 * POST: Someone has tried to connect to me.
 *****************************************************************/
inline void TCP_Server_BSD_Communicator::listen()
{
  if (std::listen(getDescriptor(), myQueueLength) < 0)
    fatal("TCP_Server_BSD_Communicator::listen(): listen failed");
}


/*****************************************************************
 * Open with string for name, port for number
 *  (for user-defined services).
 * PRE: I have been properly initialized &&
 *      service is the name of my service &&
 *      port is NOT the number of a valid TCP/IP service
 * POST: I am ready to accept connections.
 *****************************************************************/
inline void TCP_Server_BSD_Communicator::open(const string & serviceName, 
					     const unsigned short port)
{
  Server_BSD_Communicator::open(new TCP_Service(serviceName, port));
  listen();
}


/*****************************************************************
 * Construct with service-name, port
 *  (for user-defined services)
 * PRE:  service is the name of the service I am providing &&
 *        port is NOT the number of a valid TCP/IP service &&
 *        queueLength is a positive number.
 * POST: I am ready to accept connections.
 *****************************************************************/
inline TCP_Server_BSD_Communicator::TCP_Server_BSD_Communicator(
			 const string & serviceName, 
			 const unsigned short port,
			 const int queueLength)
  : Server_BSD_Communicator()
{
  setQueueLength(queueLength);
  open(serviceName, port);
}
  

/*****************************************************************
 * Close the socket's main descriptor.
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
inline void TCP_Server_BSD_Communicator::close()
{
  Server_BSD_Communicator::close();
}


/*********************************************************
 * className accessor
 * POST: RV == TCP_Server_BSD_Communicator.
 *********************************************************/
inline string TCP_Server_BSD_Communicator::className() const
{
  return "TCP_Server_BSD_Communicator";
}


/************************************************************
 * print utility (polymorphic, called by operator<<).
 * PRE: out is an ostream open for writing.
 * POST: my information has been inserted into out.
 ***********************************************************/
inline void TCP_Server_BSD_Communicator::print(ostream & out) const
{
  BSD_Communicator::print(out);
  out << ", connection descriptor: " << getDescriptor();
}



/*************************************************************
 * session descriptor extractor
 * POST: RV == sessionDescriptor 
 *  If there is a client connected to THIS communicator, we
 *  return the session descriptor for that connection, else we
 *  return the master descriptor.
 *************************************************************/
inline int TCP_Server_BSD_Communicator::getDescriptor() const
{
  return (hasConnection() 
		? myConnectionDescriptor
		: BSD_Communicator::getDescriptor() );
}

#endif
