/* TCP_Service.h simplifies the interface for services using tcp,
 *  but most of the functionality is inherited from Service.
 *
 * Author: Joel C. Adams.
 * Date: October, 1998
 * Version 0.8
 */

#ifndef TCP_SERVICE
#define TCP_SERVICE

#include "Service.h"                 // base class
#include "TCP_Protocol.h"            // TCP_Protocol abstraction
using namespace std;

class TCP_Service : public Service
{
 public:
  TCP_Service(const string & name);
  TCP_Service(const unsigned short port);
  TCP_Service(const string & name, const unsigned short port);
  TCP_Service(const unsigned short port, const string & name);
  virtual ~TCP_Service();

  virtual int BSD_Info() const;
  virtual char * TLI_Info() const;
  virtual void initProtocol();

  virtual string className() const;
 protected:
  TCP_Service();

 private:
};

/******************************************************
 * Default constructor (nothing to do)
 ******************************************************/

inline TCP_Service::TCP_Service()
{}



/******************************************************
 * Construct with TCP/IP service-name or port.
 * PRE: name is a valid TCP/IP service or a port number.
 * POST: isCopy == false and myProtocol == TCP &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline TCP_Service::TCP_Service(const string & name)
{
  initializeWith(name);   // from Service
}

/******************************************************
 * Construct with TCP/IP port.
 * PRE: port is the number of a TCP/IP (or unassigned) port.
 * POST: isCopy == false and myProtocol == TCP &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline TCP_Service::TCP_Service(const unsigned short port)
{
  initializeWith(port);  // from Service
}

/******************************************************
 * Construct with name and TCP/IP port.
 *  (for creating a new service).
 * PRE: name is the name of the new service,
 *      port is the number of an unassigned port.
 * POST: isCopy == false and myProtocol == TCP &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline TCP_Service::TCP_Service(const string & name, 
				const unsigned short port)
{
  initializeWith(name, port);  // from Service
}

/******************************************************
 * Construct with TCP/IP port and name.
 *  (for creating a new service).
 * PRE: name is the name of the new service,
 *      port is the number of an unassigned port.
 * POST: isCopy == false and myProtocol == TCP &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline TCP_Service::TCP_Service(const unsigned short port,
				const string & name)
{
  initializeWith(name, port);  // from Service
}

/********************************************************
 * Polymorphic protocol initialization method used by
 *  Service::initializeWith() to set the inherited
 *  myProtocolInfo member.
 * POST: myProtocolInfo points to a TCP_Protocol object.
 ********************************************************/

inline void TCP_Service::initProtocol()
{
  setProtocolInfo(new TCP_Protocol());
}

/*********************************************************
 * Extract the BSD information from this service's protocol
 * POST: RV == myProtocol->BSD_Info().
 *********************************************************/

inline int TCP_Service::BSD_Info() const
{
  return protocolInfo()->BSD_Info();
}

/*********************************************************
 * Extract the TLI information from this service's protocol
 * POST: RV == myProtocol->TLI_Info().
 *********************************************************/

inline char * TCP_Service::TLI_Info() const
{
  return protocolInfo()->TLI_Info();
}

/********************************************************
 * Polymorphic className() method used for diagnostics
 * POST: RV == "TCP_Service".
 ********************************************************/

inline string TCP_Service::className() const
{
  return "TCP_Service";
}

/*********************************************************
 * Destructor stub (nothing to do).
 *********************************************************/

inline TCP_Service::~TCP_Service()
{
}

#endif




