/* UDP_Client_TLI_Communicator.h defines the Communicator abstraction
 *  for UDP client sockets under TLI.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef UDP_CLIENT_TLI_TRANSPORTER
#define UDP_CLIENT_TLI_TRANSPORTER

#include "Client_TLI_Communicator.h"
#include "UDP_Service.h"
#include "TLI_UDP_SendRecv.h"
using namespace std;


class UDP_Client_TLI_Communicator : public Client_TLI_Communicator,
				    public TLI_UDP_SendRecv
{
 public:
  UDP_Client_TLI_Communicator();

  UDP_Client_TLI_Communicator(const string & serviceName,
			const string & remoteHostName);
  UDP_Client_TLI_Communicator(const unsigned short port,
			const string & remoteHostName);
  UDP_Client_TLI_Communicator(const string & serviceName, 
			const unsigned short port,
			const string & remoteHostName);
  UDP_Client_TLI_Communicator(const UDP_Client_TLI_Communicator & comm);

  void open(const string & serviceName,
	    const string & remoteHostName);
  void open(const unsigned short port,
	    const string & remoteHostName);
  void open(const string & serviceName, 
	    const unsigned short port,
	    const string & remoteHostName);

  virtual int send(const string & msg);
  virtual int receive(string & msg);

  virtual string className() const;
  UDP_Client_TLI_Communicator & 
      operator=(const UDP_Client_TLI_Communicator & comm);

  virtual ~UDP_Client_TLI_Communicator();

  virtual void connect();

 protected:
 private:
    typedef struct t_unitdata *pud_t;
    pud_t pud;
};



/****************************************************
 * Default constructor
 * POST: Inherited data members have default values.
 ****************************************************/
inline UDP_Client_TLI_Communicator::UDP_Client_TLI_Communicator()
  : Client_TLI_Communicator()
{ 
}


/***************************************************
 * open with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a UDP_Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/
inline void UDP_Client_TLI_Communicator::open(const string & serviceName,
					const string & hostName)
{
  Client_TLI_Communicator::open(new UDP_Service(serviceName), hostName);
}


/***************************************************
 * Construct with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to the Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/
inline UDP_Client_TLI_Communicator::
UDP_Client_TLI_Communicator(const string & serviceName,
		      const string & hostName)
  : Client_TLI_Communicator()
{
  open(serviceName, hostName);
}


/***************************************************
 * open with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to the UDP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/
inline void UDP_Client_TLI_Communicator::open(const unsigned short port,
					const string & hostName)
{
  Client_TLI_Communicator::open(new UDP_Service(port), hostName);
}


/***************************************************
 * Construct with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a UDP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/
inline UDP_Client_TLI_Communicator::UDP_Client_TLI_Communicator(const unsigned short port, const string & hostName)
  : Client_TLI_Communicator()
{
  open(port, hostName);
}


/***************************************************
 * open with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline void UDP_Client_TLI_Communicator::open(const string & serviceName, 
					const unsigned short port,
					const string & hostName)
{
  Client_TLI_Communicator::open(new UDP_Service(serviceName, port), 
			  hostName);
}


/***************************************************
 * Construct with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline UDP_Client_TLI_Communicator::
UDP_Client_TLI_Communicator(const string & serviceName, 
		      const unsigned short port,
		      const string & hostName)
  : Client_TLI_Communicator()
{
  open(serviceName, port, hostName);
}


/***************************************************
 * Class className accessor
 * POST: RV == "UDP_Client_TLI_Communicator".
 ***************************************************/
inline string UDP_Client_TLI_Communicator::className() const
{
  return "UDP_Client_TLI_Communicator";
}


/***************************************************
 * Destructor (nothing to do)
 ***************************************************/
inline UDP_Client_TLI_Communicator::~UDP_Client_TLI_Communicator()
{
    if (pud) t_free((char*)pud, T_UNITDATA);
}

/***************************************************
 * send a message (msg) to the server
 *   Because this is based on UDP, this should not
 *   block. However, there is no explicit command
 *   to tell it NOT to block.
 * PRE: msg contains the message to send,
 *      and the communicator has been opened.
 * POST: the return value of t_sndudata:
 *         0 if it was okay,
 *        -1 w/ t_errno set if there was a problem.
 ***************************************************/
inline int UDP_Client_TLI_Communicator::send(const string & msg) 
{
  return TLI_UDP_SendRecv::send(getDescriptor(), pud, msg);
}

/***************************************************
 * receive a message (msg) from the server
 *   This function call will block until we receive
 *   a message from the server.
 * PRE: * the communicator has been opened.
 * POST: the return value of t_sndudata:
 *         0 if it was okay,
 *        -1 w/ t_errno set if there was a problem.
 ***************************************************/
inline int UDP_Client_TLI_Communicator::receive(string & msg) 
{
  return TLI_UDP_SendRecv::receive(getDescriptor(), pud, msg);
}



#endif
