/* Winsock_Communicator.h extends the Communicator abstraction with functionality
 *   specific to Winsock sockets.
 *
 * Author: Joel Adams.
 * Date: November, 1998.
 * Version: 0.9
 *
 * Modified: Ryan VanderBijl
 * Date:     May/June, 2000
 *
 */

#ifndef Winsock_COMMUNICATOR
#define Winsock_COMMUNICATOR

#include "Communicator.h"
#include "SocketAddress.h"

// Winsock stuff:
#include <winsock2.h>
#define  WSVERS        MAKEWORD(2,0)


class Winsock_Communicator : public Communicator
{
 public:
  Winsock_Communicator();
  Winsock_Communicator(const Winsock_Communicator & comm);
  virtual ~Winsock_Communicator();

  void start(Service * service);
  virtual void stop();

  virtual int send(const string & msg);
  virtual int receive(string & msg);


  SocketAddress * addressInfo() const;

  virtual int compare(const NetworkObject & netObj) const;
  virtual void print(ostream & out) const;
  virtual string className() const;
  Winsock_Communicator & operator=(const Winsock_Communicator & comm);

 protected:
  virtual int getDescriptor() const;

  virtual void allocate();
  void setAddressInfo(SocketAddress * address);
  virtual void fatal(const char * format, ...) const;

 private:
  int myDescriptor;             // endpoint for Winsock sockets
  SocketAddress * myAddressInfo;
};


/********************************************************
 * Default constructor.
 * POST: all data members have been initialized
 *        to default values. Winsock has been initialized
 ********************************************************/
inline Winsock_Communicator::Winsock_Communicator()
  : Communicator()
{
  myDescriptor = -1;
  myAddressInfo = NULL;

  // winsock init (keeps an internal counter per call)
  WSADATA wsdata;
  if (WSAStartup(WSVERS, &wsdata))
      fatal("WSAStartup failed...\n");
}


/********************************************************
 * Open for a given service.
 * PRE: service points to a properly-defined Service.
 * POST: myServiceInfo points to service.
 ********************************************************/
inline void Winsock_Communicator::start(Service * service)
{
  Communicator::start(service);
}

/*********************************************************
 * descriptor/endpoint accessor.
 * POST: RV == myDescriptor.
 *********************************************************/
inline int Winsock_Communicator::getDescriptor() const
{
  return myDescriptor;
}


/**********************************************
 * stop(close) the socket.
 * POST: I have been stopped, and all of my
 *     dynamically-allocated memory has been reclaimed.
 **********************************************/

inline void Winsock_Communicator::stop()
{
    Communicator::stop();
    delete myAddressInfo;
    if (shutdown(getDescriptor(), SD_SEND) == SOCKET_ERROR) {
	warning("error shutting down\n");
    }
    if (closesocket(getDescriptor()) == SOCKET_ERROR) {
	warning("error closing socket \n");
    }
}

/**************************************************
 * Destructor.
 * POST: If I was started, then I am stopped.
 **************************************************/

inline Winsock_Communicator::~Winsock_Communicator()
{
  if (isOn())
    stop();

  // no more WinSocket stuff...
  WSACleanup();
}


/**********************************************************
 * Send message to remote address that sent to me most recently.
 *
 * This works for TCP and UDP clients, and TCP server (although
 *     for TCP server the getDescriptor() is overloaded).
 *     UDP Server needs to overload receive and this function so
 *     that we use sendto and recvfrom. (UDP Client works like this
 *     because the socket is already connected to the server.)
 *
 * PRE: I am started && msg is a reply to my most recent sender.
 * POST: message has been sent && RV == the number of bytes sent.
 **********************************************************/
inline int Winsock_Communicator::send(const string & msg)
{
  return ::send(getDescriptor(), msg.c_str(), msg.length(), 0);
}


/******************************************************
 * SocketAddress accessor
 * POST: RV == myAddressInfo.
 ******************************************************/
inline SocketAddress * Winsock_Communicator::addressInfo() const
{
  return myAddressInfo;
}



/***************************************************
 * Class className accessor.
 * POST: RV == Winsock_Communicator.
 ***************************************************/
inline string Winsock_Communicator::className() const
{
  return "Winsock_Communicator";
}


#endif
