#include "TCP_Server_TLI_Communicator.h"

/*****************************************************************
 * Open with string for name, port for number
 *  (for user-defined services).
 * PRE: I have been properly initialized &&
 *      service is the name of my service &&
 *      port is NOT the number of a valid TCP/IP service
 * POST: I am ready to accept connections.
 *****************************************************************/
void TCP_Server_TLI_Communicator::open(const string& serviceName,
					     const unsigned short port)
{
  myConnectionDescriptor = -1;
  Server_TLI_Communicator::open(new TCP_Service(serviceName, port));

  cli_store = cli_head = cli_tail = NULL;
  
  struct cli *t;

  // for storing a list of people trying to connect.
  //  one longer than the queue so we dont have to worry
  //  about cli_store being empty.
  for(int i = 0; i < (queueLength()+1); i++) {
     t = new struct cli; 
     if (!t) fatal("memmory not allocated for cli struct");
     t->connfd = -1;
     t->tcallp = (struct t_call*)t_alloc(getDescriptor(),T_CALL,T_ADDR);
     if (!t->tcallp) 
	  fatal("t_alloc(tcallp) failed\n");

     t->next   = cli_store;
     cli_store = t;
  }
}


/*****************************************************************
 * Close the socket's session descriptor.
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
void TCP_Server_TLI_Communicator::closeConnection()
{
  if (hasConnection())
    {
      if (std::t_close(getDescriptor())) 
	 warning("couldn'g close descriptor");

      myConnectionDescriptor = -1;
      setConnected(false);
    }
}

// for the parent server in conccurent servers to "forget" 
//  connection info which children are taking care of
void TCP_Server_TLI_Communicator::forgetConnection()
{
   if (hasConnection()) 
     {
       myConnectionDescriptor = -1;
       setConnected(false);
     }
}

/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/
TCP_Server_TLI_Communicator::~TCP_Server_TLI_Communicator()
{
  if (hasConnection())
    closeConnection();    // connection descriptor

  if (isOn())
    close();          // inherited descriptor

  if (cli_head) cli_tail->next = cli_store; /* get all at once */
  else  cli_head = cli_store;

  struct cli *t = cli_head;
  while(t) {
      t_free((char*)(cli_head->tcallp), T_CALL);
      cli_head = t;
      t = t->next;
      delete cli_head;
  }
}


/****************************************************************
 * Accept a connection.
 * PRE: I am listening for a connection.
 * POST: Someone has connected to me and 
 *        a new socket has been created for their connection and
 *        sessionDescriptor refers to this new socket.
 * Credit to: W. Richard Stevens. Parts of this function are
 *     derived from code he listed in his Unix Network Programming
 *     vol 1, 2nd Edition.
 ****************************************************************/
void TCP_Server_TLI_Communicator::acceptConnection()
{
  struct t_discon tdiscon;
  int event;

  if (hasConnection())
    closeConnection(); 

  for(;;) {
      /* if the queue is empty */
      if (!cli_head) {  /* then we need to wait for a conncetion */
	  /* get the first item in the stored list */
	  cli_head       = cli_tail = cli_store;
	  cli_store      = cli_tail->next;
	  cli_tail->next = NULL;

	  if (t_listen(getDescriptor(), cli_tail->tcallp) == -1)
	      fatal("error listening.\n");

	  /* get the info for the next item */
	  cli_tail->connfd = t_open(protocolInfo()->TLI_Info(),
				    O_RDWR, NULL);
	  if (cli_tail->connfd == -1)
	      fatal("error openning new descriptor");
	  if (t_bind(cli_tail->connfd, NULL, NULL) == -1)
	      fatal("error binding.");
      }  

      if (t_accept(getDescriptor(), cli_head->connfd, 
				    cli_head->tcallp) == 0) {
	    /* it works */
	    /* ioctl(fd, I_PUSH, "tirdwr") // DO WE NEED???? */ 

	    struct sockaddr_in remote_address;
	    memcpy(&remote_address, cli_head->tcallp->addr.buf, sizeof(remote_address));
	    setClient(SocketAddress(remote_address));
	    myConnectionDescriptor = cli_head->connfd;
	    Server_TLI_Communicator::setConnected(true);

	    // remove the entry from the connection pending list.
	    // add it to the list storage list
	    struct cli *t = cli_head;
	    cli_head      = cli_head->next;
	    t->next       = cli_store;
	    cli_store     = t;
	    if (!cli_head) cli_tail = NULL; 

	    return;
      } else if (t_errno == TLOOK) {
	  if ( (event = t_look(getDescriptor())) == T_LISTEN) {
	       /* get a new queue entry */
	       if (!cli_tail) fatal("no tail when there should be one");
	       cli_tail->next = cli_store;
	       cli_store      = cli_store->next;
	       cli_tail       = cli_tail->next;
	       cli_tail->next = NULL;

	       /* this t_listen will NOT block */
	       if (t_listen(getDescriptor(), cli_tail->tcallp) == -1)
		   fatal("error listening\n");
	       cli_tail->connfd = t_open(protocolInfo()->TLI_Info(),
					 O_RDWR, NULL);
	       if (cli_tail->connfd == -1) 
		   fatal("error opening.\n");
	       if (t_bind(cli_tail->connfd, NULL, NULL) == -1)
		   fatal("error binding");

	  } else if (event == T_DISCONNECT) {
	       if (t_rcvdis(getDescriptor(), &tdiscon))
		   fatal("error t_rcvdis");
		   
	       /* p is the previous record, t is the current */
	       struct cli *t, *p = NULL;
	       t = cli_head;
	       while(t) {
		   if (t->tcallp->sequence == tdiscon.sequence) {
		       /* if the t matches, close the connection */
		       t_close(t->connfd);
		       /* remove the entry from the list */
		       (p ? p->next : cli_head) = t->next;
		       t->next = cli_store;
		       cli_store = t;
		       if (!cli_head) cli_tail = NULL;

		       break;
		   }
		   p = t;
		   t = t->next;
	       }
	  } else {
	       fatal("unknown t_look event %d", event);
	  }
      } else {
	  fatal("unknown/unexpected t_accept error");
      }
  }
}

/*****************************************************************
 * Copy constructor
 * PRE: comm is an object which inherits TCP_Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
TCP_Server_TLI_Communicator::TCP_Server_TLI_Communicator(const TCP_Server_TLI_Communicator & comm)
   : Server_TLI_Communicator(comm)
{
  /* We do NOT want to copy pending connection information. 
   * As this information should be taken care of by the 
   * original communicator.
   */
  cli_head = cli_tail = cli_store = NULL;
  this->myConnectionDescriptor = comm.myConnectionDescriptor;
}


/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits TCP_Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
TCP_Server_TLI_Communicator& TCP_Server_TLI_Communicator::operator=(const TCP_Server_TLI_Communicator & comm)
{
  Server_TLI_Communicator::operator=(comm);

  /* We do NOT want to copy pending connection information. 
   * As this information should be taken care of by the 
   * original communicator.
   */
  cli_head = cli_tail = cli_store = NULL;

  this->myConnectionDescriptor = comm.myConnectionDescriptor;
  return *this;
}

/*****************************************************************
 * Default constructor 
 * PRE: queueLength > 0.
 * POST: server's queue length == queueLength.
 *****************************************************************/

TCP_Server_TLI_Communicator::TCP_Server_TLI_Communicator(int queueLength)
  : Server_TLI_Communicator()
{
  cli_head = cli_tail = cli_store = NULL;
  myConnectionDescriptor = -1;
  setQueueLength(queueLength);
}


/*****************************************************************
 * Construct with service-name, port
 *  (for user-defined services)
 * PRE:  service is the name of the service I am providing &&
 *        port is NOT the number of a valid TCP/IP service &&
 *        queueLength is a positive number.
 * POST: I am ready to accept connections.
 *****************************************************************/
TCP_Server_TLI_Communicator::TCP_Server_TLI_Communicator(
			 const string & serviceName,
			 const unsigned short port, 
			 const int queueLength)
  : Server_TLI_Communicator()
{
  cli_head = cli_tail = cli_store = NULL;
  setQueueLength(queueLength);
  open(serviceName, port);
}
  

/*****************************************************************
 * Close the socket's main descriptor.
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
void TCP_Server_TLI_Communicator::close()
{
  if (hasConnection()) 
      closeConnection();

  Server_TLI_Communicator::close();
}
