#include "Winsock_Communicator.h"

/********************************************************
 * Allocate the socket (endpoint), setting myDescriptor.
 * (polymorphic, called from Communicator::Open()).
 * PRE: all data members except myDescriptor are initialized.
 * POST: myDescriptor has been initialized. (an endpoint
 *       has been allocated).
 ********************************************************/

void Winsock_Communicator::allocate()
{
  // Winsock is based on BSD sockets, and uses BSD_Info for setting
  //    up the socket. There is no need for a Winsock_Info() function
  myDescriptor = ::socket(PF_INET,
			protocolInfo()->BSD_Info(),
			protocolInfo()->number());
  if (myDescriptor == INVALID_SOCKET)
    fatal("Winsock_Communicator::allocate(): call to socket failed");
}


/**********************************************************
 * receive a message via an arbitrary descriptor (utility).
 *   See note about TCP/UDP client/servers under the
 *   send(const string & msg) documentation.
 * PRE: I am open && someone is sending me a message
 *       via descriptor.
 * POST: message has been received in msg && 
 *        sender contains the address of the sender &&
 *        lastSender is a copy of sender &&
 *        RV == the number of bytes received.
 **********************************************************/
int Winsock_Communicator::receive( string & msg)
{
  const int BUF_SIZE = 1024;
  char buf[BUF_SIZE];
  msg = "";
  int total = 0;
  int count;
  // Loop until we have received the whole message.
  do
    {
      count = ::recv(getDescriptor(), buf, BUF_SIZE, 0);
      if (count == SOCKET_ERROR)
	  fatal("recv failed.");
      buf[count] = '\0';
      msg += buf;
      total += count;
    }
  while (count >= BUF_SIZE);

  return total;
}


/******************************************************
 * Address mutator
 * PRE: address is a pointer to a SocketAddress object.
 * POST: myAddressInfo == address.
 ******************************************************/
void Winsock_Communicator::setAddressInfo(SocketAddress * address)
{
  if (myAddressInfo != NULL)
    delete myAddressInfo;

  myAddressInfo = address;
}


/***************************************************
 * compare() utility function
 * (polymorphic, used by relational ops).
 * PRE: netObj is a Winsock_Communicator object.
 * POST: RV == 0 if we are the same Winsock_Communicator,
 *            +X if I am > than netObj, and
 *            -X if I am < netObj
 *            (using descriptor values).
 ***************************************************/

int Winsock_Communicator::compare(const NetworkObject & netObj) const
{
  const Winsock_Communicator * ptr 
	     = dynamic_cast<const Winsock_Communicator*>(&netObj);
  if (ptr == NULL) fatal("Winsock_Communicator::compare");

  return getDescriptor() - ptr->getDescriptor();
}

/***************************************************
 * print() utility.
 * PRE: out is an open ostream.
 * POST: my description has been inserted into out.
 ***************************************************/
void Winsock_Communicator::print(ostream & out) const
{
  Communicator::print(out);
  if (myAddressInfo) {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  << (*myAddressInfo);
  } else {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  <<  "(no address)";
  }
}


/***************************************************
 * operator= - implement the assignment operator
 * PRE: comm is a class which inherits Winsock_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
Winsock_Communicator & Winsock_Communicator::operator=(const Winsock_Communicator & comm)
{
   Communicator::operator=(comm);
   this->myDescriptor = comm.myDescriptor;
   this->myAddressInfo = comm.myAddressInfo;
   return *this;
}

/***************************************************
 * Copy Constructor
 * PRE: comm is a class which inherits Winsock_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
Winsock_Communicator::Winsock_Communicator(const Winsock_Communicator & comm)
   : Communicator(comm)
{
   this->myDescriptor = comm.myDescriptor;
   *(this->myAddressInfo) = *(comm.myAddressInfo);
}

/***************************************************
 * fatal:  overload the fatal function.
 *  This  just
 * POST: RV == Winsock_Communicator.
 ***************************************************/
inline void Winsock_Communicator::fatal(const char * format, ...) const
{
  fprintf(stderr, "\n\n*** Error: ");
  va_list args;
  va_start(args, format);
  vfprintf(stderr, format, args);
  va_end(args);
  perror("\nSystem Error Message:");
  fprintf(stderr, "Winsock Error Number: %d\n\n", GetLastError());
  WSACleanup();
  exit(1);
}
